/// <reference types="react" />
import type { RouteComponentProps } from 'react-router-dom';
import type { ISSRContext, ISSRNestContext, ISSRMidwayContext, IConfig, ISSRMidwayKoaContext } from 'ssr-types';
import type { Action } from './component';
export interface LayoutProps {
    ctx?: ISSRContext;
    config?: IConfig;
    children?: JSX.Element;
    staticList?: StaticList;
    injectState?: any;
}
export interface StaticList {
    injectCss: JSX.Element[];
    injectScript: JSX.Element[];
}
export interface ProvisionalFeRouteItem {
    path?: string;
    layout: string;
    fetch?: string;
    component?: string;
}
export interface Params<T, U> {
    ctx?: ISSRContext<T>;
    routerProps?: RouteComponentProps<U>;
    state?: any;
}
export interface ParamsMidway<T, U> {
    ctx?: ISSRMidwayContext<T>;
    routerProps?: RouteComponentProps<U>;
    state?: any;
}
export interface ParamsMidwayKoa<T, U> {
    ctx?: ISSRMidwayKoaContext<T>;
    routerProps?: RouteComponentProps<U>;
    state?: any;
}
export interface ParamsNest<T, U> {
    ctx?: ISSRNestContext<T>;
    routerProps?: RouteComponentProps<U>;
    state?: any;
}
export declare type ReactFetch<T = {}, U = {}> = (params: Params<T, U>) => Promise<any>;
export declare type ReactMidwayFetch<T = {}, U = {}> = (params: ParamsMidway<T, U>) => Promise<any>;
export declare type ReactMidwayKoaFetch<T = {}, U = {}> = (params: ParamsMidwayKoa<T, U>) => Promise<any>;
export declare type ReactNestFetch<T = {}, U = {}> = (params: ParamsNest<T, U>) => Promise<any>;
export declare type ReactESMFetch = () => Promise<{
    default: ReactFetch;
}>;
export declare type ESMLayout = () => Promise<React.FC<LayoutProps>>;
export interface StaticFC<T = {}> extends React.FC<T> {
    fetch?: ReactESMFetch;
    layoutFetch?: ReactFetch;
}
export interface DynamicFC<T = {}> extends React.FC<T> {
    (): Promise<{
        default: StaticFC<T>;
    }>;
    name: 'dynamicComponent';
    fetch?: ReactESMFetch;
    layoutFetch?: ReactFetch;
}
export declare type ReactESMFeRouteItem<T = {}, U = {}> = {
    path: string;
    fetch?: ReactESMFetch;
    component: DynamicFC<T>;
    webpackChunkName: string;
} & U;
export declare type ReactESMPreloadFeRouteItem<T = {}, U = {}> = {
    path: string;
    fetch?: ReactESMFetch;
    component: DynamicFC<T> | StaticFC<T>;
    webpackChunkName: string;
} & U;
export interface ReactRoutesType {
    Layout: React.FC<LayoutProps>;
    App?: (props: LayoutProps) => JSX.Element;
    layoutFetch: ReactFetch;
    FeRoutes: ReactESMFeRouteItem[];
    state?: any;
    reducer?: any;
}
export interface IContext<T = any> {
    state?: T;
    dispatch?: React.Dispatch<Action>;
}
//# sourceMappingURL=route.d.ts.map